package ru.amse.silina.cat.serialize;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import ru.amse.silina.cat.text.IFragment;
import ru.amse.silina.cat.text.IText;
import ru.amse.silina.cat.text.impl.DeadSpace;
import ru.amse.silina.cat.text.impl.Text;
import ru.amse.silina.cat.text.impl.TranslatedFragment;

import static ru.amse.silina.cat.serialize.ISerializationConstants.*;

public class Deserializer {

	public IText deserialize(InputStream stream)
			throws ParserConfigurationException, SAXException, IOException {
		SAXParserFactory factory = SAXParserFactory.newInstance();
		SAXParser parser = factory.newSAXParser();
		TextHandler handler = new TextHandler();
		parser.parse(stream, handler);
		return handler.getText();
	}
	
	private class TextHandler extends DefaultHandler {
		private int myFragmentStart;
		private int myFragmentEnd;
		private IText myText;
		private StringBuilder myStringBuilder;
		
		
		
		@Override
		public void startElement(String uri, String localName, String qName, Attributes attributes) {
			if (!qName.equals(LINE)) {
				myStringBuilder = new StringBuilder();
			}
			if (qName.equals(TRANSLATED) || (qName.equals(DEADSPACE))) {
				myFragmentStart = Integer.parseInt(attributes.getValue(START));
				myFragmentEnd = Integer.parseInt(attributes.getValue(END));
			} else if (qName.equals(LINE)) {
				myStringBuilder.append("\n\r");
			}
		}
		
		@Override
		public void characters(char[] ch, int start, int length) {
				myStringBuilder.append(ch, start, length);
		}
		
		@Override 
		public void endElement(String uri, String localName, String qName) {
			if (qName.equals(TEXT)) {
				myText = new Text(myStringBuilder.toString());
			} else if (qName.equals(DEADSPACE)) {
				IFragment fragment = new DeadSpace(myFragmentStart, myFragmentEnd, myText);
				myText.addFragment(fragment);
			} else if (qName.equals(TRANSLATED)) {
				IFragment fragment = new TranslatedFragment(myFragmentStart, myFragmentEnd,
						myStringBuilder.toString(), myText);
				myText.addFragment(fragment);
			}
			if (!qName.equals(LINE)) {
				myStringBuilder = new StringBuilder();
			}
		}
		
		public IText getText() {
			return myText;
		}
	}
}
