package ru.amse.silina.cat.serialize;

import java.io.OutputStream;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import ru.amse.silina.cat.text.IFragment;
import ru.amse.silina.cat.text.IText;
import ru.amse.silina.cat.text.impl.DeadSpace;
import ru.amse.silina.cat.text.impl.TranslatedFragment;

import static ru.amse.silina.cat.serialize.ISerializationConstants.*;

public class Serializer {
	
	public void serialize(IText text, OutputStream stream) throws XMLStreamException {
		
		XMLOutputFactory factory = XMLOutputFactory.newInstance();
		XMLStreamWriter writer = factory.createXMLStreamWriter(stream, "utf8");
		try {
			writer.writeStartDocument();
			writer.writeStartElement(ITEXT);
			writer.writeStartElement(TEXT);
			
			int start = 0;
			int next = 0;
			String source = text.getSourceText();
			
			while (next != -1) {
				next = source.indexOf('\n', start);
				String line;
				if (next != -1) {
					line = source.substring(start, next - 1);
					writer.writeEmptyElement(LINE);
				} else {
					line = source.substring(start, source.length());
				}
				writer.writeCharacters(line);
				start = next + 1;
			}
			
			writer.writeEndElement();
			writer.writeStartElement(FRAGMENTS);
			for (IFragment fragment : text.getFragments()) {
				if (fragment instanceof DeadSpace) {
					writer.writeEmptyElement(DEADSPACE);
				} else {
					writer.writeStartElement(TRANSLATED);
				}
				writer.writeAttribute(START, fragment.getStart() + "");
				writer.writeAttribute(END, fragment.getEnd() + "");
				if (fragment instanceof TranslatedFragment) {
					writer.writeCharacters(((TranslatedFragment)fragment).getTranslation());
					writer.writeEndElement();
				}
			}
			writer.writeEndElement();
			writer.writeEndElement();
			writer.writeEndDocument();
		} finally {
			writer.close();
		}
	}
}
