package ru.amse.silina.cat.text.impl;

import java.util.ListIterator;

import ru.amse.silina.cat.text.IFragment;
import ru.amse.silina.cat.text.IText;

public abstract class Fragment implements IFragment {
	protected static final String NEGATIVE_FRAGMENT_LENGTH_ERROR_MESSAGE = 
		"End of a fragment must not be less or equal to start";
	protected static final String NEGATIVE_FRAGMENT_START_ERROR_MESSAGE = 
		"Start of a fragment must not be negative";
	protected static final String FRAGMENTS_INTERSECT_ERROR_MESSAGE = 
		"Fragments must not intersect";
	protected static final String END_OF_FRAGMENT_OUTSIDE_THE_TEXT_ERROR_MESSAGE = 
		"End of fragment must not be more than the source text length";
	
	private int myStart;
	private int myEnd;
	private final IText myText;
	
	public Fragment(int start, int end, IText text) {
		myStart = start;
		myEnd = end;
		myText = text;
		checkFragmentEdges(start, end);
	}

	private void checkFragmentEdges(int start, int end) {
		if (start < 0) {
			throw new IllegalArgumentException(NEGATIVE_FRAGMENT_START_ERROR_MESSAGE);
		}
		if (start >= end) {
			throw new IllegalArgumentException(NEGATIVE_FRAGMENT_LENGTH_ERROR_MESSAGE);
		}
		if (end > myText.getSourceText().length()) {
		}
	}
	
	@Override
	public int getStart() {
		return myStart;
	}

	@Override
	public int getEnd() {
		return myEnd;
	}
	
	@Override
	public void setStart(int start) {
		checkFragmentEdges(start, myEnd);
		for (ListIterator<IFragment> iter = myText.getFragments().listIterator(); iter.hasNext(); ) {
			if (iter.next() == this) {
				iter.previous();
				if (!iter.hasPrevious()) {
					break;
				}
				IFragment previous = iter.previous();
				if (previous.getEnd() >= start) {
					throw new IllegalArgumentException(FRAGMENTS_INTERSECT_ERROR_MESSAGE);
				} 
				break;
			}
		}
		myStart = start;
	}
	
	@Override
	public void setEnd(int end) {
		checkFragmentEdges(myStart, end);
		for (ListIterator<IFragment> iter = myText.getFragments().listIterator(); iter.hasNext(); ) {
			if (iter.next() == this) {
				if (!iter.hasNext()) {
					break;
				}
				IFragment next = iter.next();
				if (next.getStart() <= end) {
					throw new IllegalArgumentException(FRAGMENTS_INTERSECT_ERROR_MESSAGE);
				}
				break;
			}
		}
		myEnd = end;
	}
}
