package ru.amse.silina.cat.text.impl;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import ru.amse.silina.cat.text.IFragment;
import ru.amse.silina.cat.text.IText;
import ru.amse.silina.cat.text.ITranslatedFragment;

public class Text implements IText {
	private final String mySourceText;
	private final List<IFragment> myFragments;
	
	public Text(String sourceText) {
		mySourceText = sourceText;
		myFragments = new LinkedList<IFragment>();
	}

	@Override
	public String getSourceText() {
		return mySourceText;
	}

	@Override
	public String getTargetText() {
		StringBuilder target = new StringBuilder();
		int beginIndex = 0;
		for (IFragment fragment : myFragments) {
			if (fragment instanceof ITranslatedFragment) {
				int start = fragment.getStart();
				if (start > 0) {
					target.append(mySourceText.substring(beginIndex, start));
				}
				target.append(((ITranslatedFragment)fragment).getTranslation());
				beginIndex = fragment.getEnd() + 1;
			}
		}
		target.append(mySourceText.substring(beginIndex, mySourceText.length()));
		return target.toString();
	}

	@Override
	public List<IFragment> getFragments() {
		return Collections.unmodifiableList(myFragments);
	}
	
	@Override
	public void addFragment(IFragment newFragment) {
		if ((myFragments.size() == 0) || 
				((myFragments.size() > 0) && (newFragment.getEnd() < myFragments.get(0).getStart()))) {
			myFragments.add(0, newFragment);
			return;
		}
		int firstAfter = 0;
		for (IFragment fragment : myFragments) {
			if (newFragment.getStart() < fragment.getStart()) {
				break;
			}	
			firstAfter++;
		}
		if ((firstAfter > 0) && 
				(myFragments.get(firstAfter - 1).getEnd() >= newFragment.getStart())) {
			throw new IllegalArgumentException();
		}
		myFragments.add(firstAfter, newFragment);
	}


	@Override 
	public IFragment getFragment(int symbol) {
		IFragment ownerFragment = null;
		for (IFragment fragment : myFragments) {
			if (fragment.getStart() > symbol) {
				break;
			}
			ownerFragment = fragment;
		}
		if ((ownerFragment != null) && (ownerFragment.getEnd() >= symbol)) {
			return ownerFragment;
		} else {
			return null;
		}
	}
	
	@Override
	public boolean removeFragment(IFragment fragment) {
		return myFragments.remove(fragment);
	}
}
