package ru.amse.silina.cat.ui.fragmentspane;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import javax.swing.JComponent;
import javax.swing.JFrame;

import ru.amse.silina.cat.text.IDeadSpace;
import ru.amse.silina.cat.text.IFragment;
import ru.amse.silina.cat.text.IText;
import ru.amse.silina.cat.text.ITranslatedFragment;

public class JFragmentsPane extends JComponent {
	public static final int SHIFT = 2;
	private IText myText;
	private IUICalculator myCounter;
	private List<Integer> myFirstInLine;
	private IFragmentDataProvider myFragmentDataProvider;
	private List<IFragmentClickListener> myFragmentClickListeners = new ArrayList<IFragmentClickListener>();
	private List<IFragmentMarkListener> myFragmentMarkListeners = new ArrayList<IFragmentMarkListener>();
	private IFragment myLastClicked = null;
	private AbstractFragment mySelection = null;
	private int myInitialOffset = -1; 
	

	public JFragmentsPane(JFrame frame, IText text,
			IFragmentDataProvider fragmentDataProvider) {
		myText = text;
		myCounter = new UICalculator(this);
		myFragmentDataProvider = fragmentDataProvider;
		setPreferredSize(new Dimension(250, 500));
		setMinimumSize(new Dimension(50, 50));
		this.addMouseListener(new FragmentClickNotifier());
		this.addMouseMotionListener(new FragmentClickNotifier());
	}

	public IText getText() {
		return myText;
	}
	
	public List<Integer> getFirstInLine() {
		return myFirstInLine;
	}
	
	public void loadText(IText text, IFragmentDataProvider fragmentDataProvider) {
		myText = text;
		myFragmentDataProvider = fragmentDataProvider;
		repaint();
	}

	public IUICalculator getCounter() {
		return myCounter;
	}

	public IFragmentDataProvider getFragmentDataProvider() {
		return myFragmentDataProvider;
	}
	
	public void removeSelection() {
		mySelection = null;
	}

	@Override
	public void paintComponent(Graphics g) {
		myFirstInLine = myCounter.buildFirstInLine(g);
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());
		paintFragments(g);
		paintText(g);
	}

	private void paintText(Graphics g) {
		String output = myFragmentDataProvider.getText();
		g.setColor(Color.BLACK);
		int lineHeight = g.getFontMetrics().getHeight();
		for (int i = 0; i < myFirstInLine.size() - 1; i++) {
			g.drawString(output.substring(myFirstInLine.get(i), myFirstInLine
					.get(i + 1)), 0, lineHeight * (i + 1));
		}
		g.drawString(output.substring(myFirstInLine.get(myFirstInLine.size() - 1)),
				0, lineHeight * myFirstInLine.size());
	}

	private void paintFragments(Graphics g) {
		for (IFragment fragment : myText.getFragments()) {
			paintFragment(g, fragment);
		}
	}

	private void paintFragment(Graphics g, IFragment fragment) {
		Color polyColor;
		Color polyFrameColor;
		if (fragment == myLastClicked) {
			polyFrameColor = Color.RED;
		} else {
			polyFrameColor = Color.BLACK;
		}
		if (fragment instanceof IDeadSpace) {
			polyColor = Color.YELLOW;
			polyFrameColor = Color.YELLOW;
		} else if (fragment instanceof ITranslatedFragment) {
			polyColor = Color.GREEN;
		} else {
			polyColor = Color.LIGHT_GRAY;
			polyFrameColor = Color.LIGHT_GRAY;
		}
		FragmentArea position = getFragmentPosition(g, myFragmentDataProvider.getStart(fragment), 
				myFragmentDataProvider.getEnd(fragment));
		drawPoly(g, position, polyColor, polyFrameColor);
	}

	private FragmentArea getFragmentPosition(Graphics g, int start, int end) {
		int lineHeight = g.getFontMetrics().getHeight();
		int currentLine = myCounter.getLineByOffset(myFirstInLine, start);
		int yTop = lineHeight * currentLine + SHIFT;
		String leftShift = myFragmentDataProvider.getText().substring(
				myFirstInLine.get(currentLine), start);
		int xLeft = g.getFontMetrics().stringWidth(leftShift);
		int startLine = currentLine;
		currentLine = myCounter.getLineByOffset(myFirstInLine, end);
		int yBottom = lineHeight * currentLine + SHIFT;
		String rightShift = myFragmentDataProvider.getText().substring(myFirstInLine.get(currentLine), end + 1);
		int xRight = g.getFontMetrics().stringWidth(rightShift);
		if (startLine == currentLine) {
			int[] yPoints = {yTop, yTop, yBottom + lineHeight,
					yBottom + lineHeight };
			int[] xPoints = {xLeft, xRight, xRight, xLeft };
			int[][] coordinates = { xPoints, yPoints };
			return new FragmentArea(coordinates);
		} else if ((currentLine == startLine + 1) && (xRight >= xLeft)) {
			int[] yPoints1 = {yTop, yTop + lineHeight, yTop + lineHeight, yTop };
			int[] xPoints1 = {xLeft, xLeft, this.getWidth(), this.getWidth() };
			int[] yPoints2 = {yBottom, yBottom, yBottom + lineHeight,
					yBottom + lineHeight };
			int[] xPoints2 = {0, xRight, xRight, 0 };
			int[][] coordinates = {xPoints1, yPoints1, xPoints2, yPoints2 };
			return new FragmentArea(coordinates);
		} else {
			int[] yPoints = {yTop, yTop, yBottom, yBottom,
					yBottom + lineHeight, yBottom + lineHeight,
					yTop + lineHeight, yTop + lineHeight };
			int[] xPoints = {xLeft, this.getWidth(), this.getWidth(), xRight,
					xRight, 0, 0, xLeft };
			int[][] coordinates = {xPoints, yPoints };
			return new FragmentArea(coordinates);
		}
	}

	private void drawPoly(Graphics g, FragmentArea position,
			Color polyColor, Color polyFrameColor) {
		g.setColor(polyColor);
		g.fillPolygon(position.getX1(), position.getY1(), position.getLength1());
		if (position.getPolygonsCount() == 2) {
			g.fillPolygon(position.getX2(), position.getY2(), position.getLength2());
		}
		
		g.setColor(polyFrameColor);
		g.drawPolygon(position.getX1(), position.getY1(), position.getLength1());
		if (position.getPolygonsCount() == 2) {
			g.drawPolygon(position.getX2(), position.getY2(), position.getLength2());
		}
	}

	public void setSelectedFragment(IFragment fragment) {
		myLastClicked = fragment;
	}
	
	public IFragment getSelectedFragment() {
		return myLastClicked;
	}

	public void addFragmentClickListener(IFragmentClickListener l) {
		myFragmentClickListeners.add(l);
	}
	
	public void addFragmentMarkListener(IFragmentMarkListener l) {
		myFragmentMarkListeners.add(l);
	}

	public void removeFragmentClickListener(IFragmentClickListener l) {
		myFragmentClickListeners.remove(l);
	}

	public void removeFragmentMarkListener(IFragmentMarkListener l) {
		myFragmentMarkListeners.remove(l);
	}
	
	protected void fireFragmentClicked(IFragment fragment) {
		for (IFragmentClickListener listener : myFragmentClickListeners) {
			listener.fragmentClicked(fragment);
		}
	}

	protected void fireFragmentMarked(int start, int end) {
		for (IFragmentMarkListener listener : myFragmentMarkListeners) {
			listener.fragmentMarked(start, end);
		}
	}
	
	private class FragmentClickNotifier extends MouseAdapter {
		
		@Override
		public void mousePressed(MouseEvent e) {
			IUICalculator counter = ((JFragmentsPane) e.getComponent()).getCounter();
			IFragment fragment = counter.seekFragment(e.getX(), e.getY(), getGraphics());
			if (fragment == null) {
				myInitialOffset = counter.getOffsetByCoordinates(e.getX(), e.getY(), getGraphics());
			} else {
				fireFragmentClicked(fragment);
			}
		}

		@Override
		public void mouseDragged(MouseEvent e) {
			if (myInitialOffset != -1) {
				IUICalculator counter = ((JFragmentsPane)e.getComponent()).getCounter();
				int offset = getCutOffset(e, counter);
				myText.removeFragment(mySelection);
				myFragmentDataProvider.recalculateFragmentPositions();
				if (myInitialOffset > offset) {
					mySelection = new AbstractFragment(getTargetOffset(offset), getTargetOffset(myInitialOffset));
				} else {
					mySelection = new AbstractFragment(getTargetOffset(myInitialOffset), getTargetOffset(offset));
				}
				myText.addFragment(mySelection);
				myFragmentDataProvider.recalculateFragmentPositions();
				repaint();
			}
		}

		private int getCutOffset(MouseEvent e, IUICalculator counter) {
			int offset = counter.getOffsetByCoordinates(e.getX(), e.getY(), getGraphics());
			if (offset > myInitialOffset) {
				int cutOffset = counter.seekClosestFragmentOffsetForward(myInitialOffset); 
				if ((cutOffset != -1) && (cutOffset <= offset)) {
					offset = cutOffset - 1;
				}
			} else {
				int cutOffset = counter.seekClosestFragmentOffsetBack(myInitialOffset); 
				if (cutOffset >= offset) {
					offset = cutOffset + 1;
				}
			}
			return offset;
		}
		
		private int getTargetOffset(int offset) {
			int shift = 0;
			for (ListIterator<IFragment> i = myText.getFragments().listIterator(myText.getFragments().size()); i.hasPrevious(); ) {
				IFragment f = i.previous();
				if (myFragmentDataProvider.getEnd(f) < offset) {
					shift = myFragmentDataProvider.getEnd(f) - f.getEnd();
					break;
				}
			}
			return offset - shift;
		}
		
		@Override
		public void mouseReleased(MouseEvent e) {
			if (myInitialOffset != -1) {
				myText.removeFragment(mySelection);
				mySelection = null;
				myFragmentDataProvider.recalculateFragmentPositions();
				IUICalculator counter = ((JFragmentsPane)e.getComponent()).getCounter();
				int offset = getCutOffset(e, counter);
				if (offset != myInitialOffset) {
					fireFragmentMarked(getTargetOffset(myInitialOffset), getTargetOffset(offset));
					myInitialOffset = -1;
				}
			}
		}
	}
	
}
